# API Middleware
# Originally created by Shawn Clake
# shawn.clake@gmail.com
# May 2018
#
# Contains Scriptax and Commandtax
#
# This application is used to automate API processes using arbitrary API's
# It includes a CLI as well as web interface
# However the web interface will need to be customized per API usage


# Default imports
import click

# Application imports
from .ah.Connector import Connector
from .ah.web.WebServer import *
from .config.Config import Config as ConfigConsumer
from .grammar.grammartest import GrammarTest
from .utilities.Arguments import SystemCommands


class Apitax:

    # Entry point of the program
    def apitax(self, args):
        # General procedure is as follows
        # Sets up logical defaults for parameters
        # Checks config for any overrides to those params
        # Checks cli params for any overrides to those params
        # CLI > Config > Default
        # args = sys.argv[1:]

        usage = 'cli'
        debug = False
        sensitive = False
        username = ''
        password = ''
        command = ''
        script = ''

        config = ConfigConsumer()

        if (config.has('default-username')):
            username = config.get('default-username')

        if (config.has('default-password')):
            password = config.get('default-password')

        if (config.has('default-mode')):
            usage = config.get('default-mode')

        print("Automation Haven")

        if ('--cli' in args):
            usage = 'cli'
        elif ('--web' in args):
            usage = 'web'
        elif('--grammar-test' in args):
            usage = 'grammar-test'

        if ('--debug' in args):
            debug = True

        if ('--sensitive' in args):
            sensitive = True

        if ('-u' in args):
            username = args[args.index('-u') + 1]

        if ('-p' in args):
            password = click.prompt('Enter Your Password', hide_input=True)

        if ('-s' in args):
            script = args[args.index('-s') + 1]

        if ('-r' in args):
            command = args[args.index('-r') + 1]
        elif (script == '' and usage == 1):
            command = click.prompt('R')

        # This is to turn the '-s' flag into a command behind the scenes
        if (script != ''):
            command = 'script ' + script

        if (usage == 'cli'):
            # Authentication is incorporated into Connector
            connector = Connector(debug=debug, sensitive=sensitive, command=command, username=username, password=password,
                                  json=True)
            result = connector.execute()
            print(json.dumps(result.getRequest().data.dataStore))
            # print(result.getRequest().data.getData('5.3.role_assignments.0.links.assignment'))

        elif (usage == 'web'):
            bSrv = bottleServer()
            bSrv.start(config.get("ip"), config.get("port"))

        elif(usage == 'grammar-test'):
            GrammarTest(script)




# System import
import json
import re

# Library import
import requests

# Application import
from log import Log


# Request is the 'legs' of the application.
# It is responsible to act as a facade to the 'requests' library
# This provides additional output, logging, and logic capabilities
class Request:
    def __init__(self, url, headers='', postData='', paramData={}, pathData={}, debug=False, sensitive=False,
                 customResponse=False):
        self.url = url
        self.headers = headers
        self.postData = postData
        self.paramData = paramData
        self.pathData = pathData
        self.debug = debug
        self.sensitive = sensitive
        self.request = None
        self.customResponse = customResponse
        self.log = Log('logs/log.log')

    def setDebug(self, debug):
        self.debug = debug

    def setSensitive(self, sensitive):
        self.sensitive = sensitive

    def setHeaders(self, headers):
        self.headers = headers

    def setUrl(self, url):
        self.url = url

    def setPostData(self, postData):
        self.postData = postData

    def setParamData(self, paramData):
        self.paramData = paramData

    def setPathData(self, pathData):
        self.pathData = pathData

    def getRequestUrl(self):
        return self.request.url

    def getResponse(self):
        return self.request

    def getResponseHeaders(self):
        if (self.customResponse):
            return self.request['headers']
        else:
            return self.request.headers

    def getResponseBody(self):
        if (self.customResponse):
            return self.request['text']
        else:
            return self.request.text

    def getResponseBodyAsStructure(self):
        return json.loads(self.getResponseBody())

    def getResponseStatusCode(self):
        if (self.customResponse):
            return self.request['status_code']
        else:
            return self.request.status_code

    def printCLIResponse(self):
        self.log.log(json.dumps(self.getResponseBodyAsStructure(), indent=2, separators=(',', ': ')))

    def printDebugResponse(self):
        self.log.log('Status: ' + str(self.getResponseStatusCode()))
        self.log.log('Headers:')
        if (self.sensitive):
            self.log.log('Headers are not shown as it contains sensitive data. ie. token')
        else:
            self.log.log(self.getResponseHeaders())
        self.log.log('Body:')
        self.log.log(self.getResponseBody())

    def printDebugRequest(self):
        self.log.log('Endpoint:        ' + self.url)
        self.log.log('Formed Endpoint: ' + self.getRequestUrl())
        self.log.log('Headers:')
        if (self.sensitive):
            self.log.log('Headers are not shown as it contains sensitive data. ie. password')
        else:
            self.log.log(self.headers)
        self.log.log('Post Data:')
        if (self.sensitive):
            self.log.log('Post Data is not shown as it contains sensitive data. ie. password')
        elif (not self.postData):
            self.log.log('{}')
        else:
            self.log.log(self.postData)

    def injectPathData(self):
        if (not self.pathData):
            return
        matches = re.findall('{[A-z0-9]{1,}}', self.url)
        for match in matches:
            matchStr = match[1:-1]
            if (self.pathData.has_key(matchStr)):
                self.url = self.url.replace(match, self.pathData.get(matchStr))
            else:
                self.log.log('Path data did not contain key for `' + matchStr + '`')

    def logRequest(self):
        if (self.debug):
            self.log.log('')
            self.log.log('<==========')
            self.printDebugRequest()

            self.log.log('')
            self.printDebugResponse()
            self.log.log('==========>')
            self.log.log('')
            self.log.log('')
        else:
            self.log.log('')
            self.log.log('<==========')
            self.printCLIResponse()
            self.log.log('==========>')
            self.log.log('')
            self.log.log('')

    def post(self):
        self.injectPathData()
        self.request = requests.post(self.url, data=json.dumps(self.postData), headers=self.headers,
                                     params=self.paramData)
        self.logRequest()
        # if(self.debug):
        #  self.log.log('')
        #  self.log.log('<==========')
        #  self.printDebugRequest()

        #  self.log.log('')
        #  self.printDebugResponse()
        #  self.log.log('==========>')
        #  self.log.log('')
        #  self.log.log('')
        # else:
        #  self.log.log('')
        #  self.log.log('<==========')
        #  self.printCLIResponse()
        #  self.log.log('==========>')
        #  self.log.log('')
        #  self.log.log('')

    def get(self):
        self.injectPathData()
        self.request = requests.get(self.url, data=json.dumps(self.postData), headers=self.headers,
                                    params=self.paramData)
        self.logRequest()
        # if(self.debug):
        #  self.log.log('')
        #  self.log.log('<==========')
        #  self.printDebugRequest()

        #  self.log.log('')
        #  self.printDebugResponse()
        #  self.log.log('==========>')
        #  self.log.log('')
        #  self.log.log('')
        # else:
        #  self.log.log('')
        #  self.log.log('<==========')
        #  self.printCLIResponse()
        #  self.log.log('==========>')
        #  self.log.log('')
        #  self.log.log('')

    def put(self):
        self.injectPathData()
        self.request = requests.put(self.url, data=json.dumps(self.postData), headers=self.headers,
                                    params=self.paramData)
        self.logRequest()

    def patch(self):
        self.injectPathData()
        self.request = requests.patch(self.url, data=json.dumps(self.postData), headers=self.headers,
                                      params=self.paramData)
        self.logRequest()

    def delete(self):
        self.injectPathData()
        self.request = requests.delete(self.url, data=json.dumps(self.postData), headers=self.headers,
                                       params=self.paramData)
        self.logRequest()



# System import
import shlex

# Application import
from ahRequests.script import Script
from ahRequests.custom import Custom
from ahRequests.request import Request
from ahRequests.driverCommandsFactory import DriverCommandsFactory


# Command is used to distribute the workload amoung a heirarchy of possible handlers
# Command is the 'brain' of the application
class Command:
    def __init__(self, header, command, config, debug=False, sensitive=False):

        if (type(command) is not list):
            command = shlex.split(command.strip())
        if (len(command) < 1):
            return
        self.request = None
        if (command[0] == 'script'):
            self.request = Script(header, debug, sensitive)
            self.request.setConfig(config)
        elif (command[0] == 'custom'):
            self.request = Custom(header, debug, sensitive)
        else:
            customCommands = DriverCommandsFactory.make(config.get('driver') + 'Commands')

            self.request = customCommands.handle(header, command, debug, sensitive)
        self.request.handle(command[1:])

    def getRequest(self):
        return self.request



# Generated from D:/Programming/Projects/NoPatience/API-Middleware/grammar/src\test1.g4 by ANTLR 4.7
from antlr4 import *
from io import StringIO
from typing import TextIO
import sys


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\2\5")
        buf.write("\24\b\1\4\2\t\2\4\3\t\3\4\4\t\4\3\2\3\2\3\3\6\3\r\n\3")
        buf.write("\r\3\16\3\16\3\4\3\4\3\4\3\4\2\2\5\3\3\5\4\7\5\3\2\3\3")
        buf.write("\2\62;\2\24\2\3\3\2\2\2\2\5\3\2\2\2\2\7\3\2\2\2\3\t\3")
        buf.write("\2\2\2\5\f\3\2\2\2\7\20\3\2\2\2\t\n\7-\2\2\n\4\3\2\2\2")
        buf.write("\13\r\t\2\2\2\f\13\3\2\2\2\r\16\3\2\2\2\16\f\3\2\2\2\16")
        buf.write("\17\3\2\2\2\17\6\3\2\2\2\20\21\7\"\2\2\21\22\3\2\2\2\22")
        buf.write("\23\b\4\2\2\23\b\3\2\2\2\4\2\16\3\b\2\2")
        return buf.getvalue()


class test1Lexer(Lexer):
    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [DFA(ds, i) for i, ds in enumerate(atn.decisionToState)]

    T__0 = 1
    NUMBER = 2
    WHITESPACE = 3

    channelNames = [u"DEFAULT_TOKEN_CHANNEL", u"HIDDEN"]

    modeNames = ["DEFAULT_MODE"]

    literalNames = ["<INVALID>",
                    "'+'", "' '"]

    symbolicNames = ["<INVALID>",
                     "NUMBER", "WHITESPACE"]

    ruleNames = ["T__0", "NUMBER", "WHITESPACE"]

    grammarFileName = "test1.g4"

    def __init__(self, input=None, output: TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.7")
        self._interp = LexerATNSimulator(self, self.atn, self.decisionsToDFA, PredictionContextCache())
        self._actions = None
        self._predicates = None



# System import
import json

# Library import
from bottle import route, run, template, request, static_file, post, Bottle

# Application import
from connector import Connector


# from command import Command
# from ahRequests.authentication import *


# BottleServer is used to provide a web interface which routes through Connector
# It is also used to serve css, html, and javascript

@route('/')
def index():
    return static_file('index.html', 'webSrv/pages/')


# Hosts html file which will be invoked from browser.
@route('/pages/<staticFile>')
def serve_static_file(staticFile):
    filePath = 'webSrv/pages/'
    return static_file(staticFile, filePath)


# host css files which will be invoked implicitly by your html files.
@route('/pages/css/<cssFile>')
def serve_css_files(cssFile):
    filePath = 'webSrv/pages/css/'
    return static_file(cssFile, filePath)


# host js files which will be invoked implicitly by your html files.
@route('/pages/js/<jsFile>')
def serve_js_files(jsFile):
    filePath = 'webSrv/pages/js/'
    return static_file(jsFile, filePath)


# Authentication endpoint is used to facilitate simpler authentication
@route('/api/auth/', method='POST')
def execute_api_auth():
    connector = Connector(sensitive=True, username=request.json['user'], password=request.json['pass'])

    if (connector.http.isTokenable()):
        return json.dumps({"status": 201, "auth": "presumed success", "token": connector.token,
                           "body": json.loads(connector.auth.getResponseBody())})
    else:
        return json.dumps({"status": 400,
                           "auth": "authentication does not support tokens, please pass username and password with each API request"})


# Command endpoint is used to facilitate simpler requests
@route('/api/command/', method='POST')
def execute_api_command():
    connector = None

    if ('token' in request.json):
        connector = Connector(token=request.json['token'], command=request.json['command'],
                              debug=request.json['debug'], sensitive=request.json['sensitive'])
    else:
        connector = Connector(username=request.json['user'], password=request.json['pass'],
                              command=request.json['command'], debug=request.json['debug'], sensitive=True)

    commandHandler = connector.execute()

    return json.dumps({"status": commandHandler.getRequest().getResponseStatusCode(),
                       "body": json.loads(commandHandler.getRequest().getResponseBody())})


@route('/dummy/<filename:path>')
def serve_dummy(filename):
    return static_file(filename, root='/directory/to/files')


class bottleServer():

    def start(self, ip, port):
        # self.app = Bottle()
        run(host=ip, port=port, reloader=True, debug=True)




from command import Command
from ahRequests.authentication import *
from httpPlug import HttpPlug
from httpPlugFactory import HttpPlugFactory
from headerBuilder import HeaderBuilder
from config import Config


# The 'heart' of the application
# Connector facilitates the initialization of the connection to an API
# Connector handles setting up the driver, authetnication, headers, and then
# using all of that to execute a command
#
# Additional interfaces to this utility should directly communicate with connector
# and likely nothing else. Connector handles the rest.
class Connector:

    def __init__(self, debug=False, sensitive=False, command='', username='', password='', token='', json=True):
        self.debug = debug
        self.sensitive = sensitive
        self.command = command
        self.username = username
        self.password = password
        self.token = token
        self.commandHandler = None

        self.config = Config()
        self.http = HttpPlugFactory.make(self.config.get('driver') + 'Driver')

        self.auth = AuthRequest(self.username, self.password, self.http, self.debug, self.config)

        self.header = HeaderBuilder()
        if (json):
            self.header.build(self.http.getContentTypeJSON())

        if (token == ''):
            # self.auth = AuthRequest(self.username, self.password, self.http, self.debug, self.config)
            preHeader = self.header.header.copy()
            self.header.build(self.http.getPasswordAuthHeader(self.username, self.password))
            if (self.http.isTokenable()):
                self.auth.authenticate()
                self.token = self.auth.getToken()
                self.header.header = preHeader
                self.header.build(self.http.getTokenAuthHeader(self.token))
        else:
            self.header.build(self.http.getTokenAuthHeader(self.token))

        # print(self.token)

    def execute(self, command=''):
        if (command != ''):
            self.command = command
        self.commandHandler = Command(self.header, self.command, self.config, debug=self.debug,
                                      sensitive=self.sensitive)
        return self.commandHandler



# System import
import os
import logging
from logging.handlers import RotatingFileHandler

# Logging object to prevent duplicate handler adding
#global loggers
loggers = {}



# Handles printing to CLI as well as printing to log file
class Log:
    def __init__(self, logFile):
        # logging.basicConfig(filename=filepath,level=logging.INFO)
        # log_formatter = logging.Formatter('%(asctime)s %(levelname)s %(funcName)s(%(lineno)d) %(message)s')

        if (not loggers.get('main')):
            log_formatter = logging.Formatter('%(asctime)s %(levelname)s  %(message)s')

            # Each log file can be up to 25MB big and keeps a max of 3 backup files. Thus, the largest size of the log folder is 100MB
            my_handler = RotatingFileHandler(logFile, mode='a', maxBytes=25 * 1024 * 1024,
                                             backupCount=3, encoding=None, delay=0)
            my_handler.setFormatter(log_formatter)
            my_handler.setLevel(logging.INFO)

            app_log = logging.getLogger('root')
            app_log.setLevel(logging.INFO)

            app_log.addHandler(my_handler)
            loggers.update({'main': app_log})

    def log(self, text):
        # logging.info(' '+text)
        loggers.get('main').info(text)
        print(text)

    def error(self, text):
        # logging.info(' '+text)
        loggers.get('main').info('### Error: ' + text)
        print('### Error: ' + text)



# Base class for driver command classes
class DriverCommands(object):

    def handle(self, header, command, debug=False, sensitive=False):
        self.header = header
        self.command = command
        self.debug = debug
        self.sensitive = sensitive
        self.request = None

    def getRequest(self):
        return self.request



# Generated from D:/Programming/Projects/Apitax/grammar/src\Ah2.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .Ah2Parser import Ah2Parser
else:
    from Ah2Parser import Ah2Parser

# This class defines a complete listener for a parse tree produced by Ah2Parser.
class Ah2Listener(ParseTreeListener):

    # Enter a parse tree produced by Ah2Parser#prog.
    def enterProg(self, ctx:Ah2Parser.ProgContext):
        pass

    # Exit a parse tree produced by Ah2Parser#prog.
    def exitProg(self, ctx:Ah2Parser.ProgContext):
        pass


    # Enter a parse tree produced by Ah2Parser#statements.
    def enterStatements(self, ctx:Ah2Parser.StatementsContext):
        pass

    # Exit a parse tree produced by Ah2Parser#statements.
    def exitStatements(self, ctx:Ah2Parser.StatementsContext):
        pass


    # Enter a parse tree produced by Ah2Parser#statement.
    def enterStatement(self, ctx:Ah2Parser.StatementContext):
        pass

    # Exit a parse tree produced by Ah2Parser#statement.
    def exitStatement(self, ctx:Ah2Parser.StatementContext):
        pass


    # Enter a parse tree produced by Ah2Parser#expr.
    def enterExpr(self, ctx:Ah2Parser.ExprContext):
        pass

    # Exit a parse tree produced by Ah2Parser#expr.
    def exitExpr(self, ctx:Ah2Parser.ExprContext):
        pass


    # Enter a parse tree produced by Ah2Parser#set_var.
    def enterSet_var(self, ctx:Ah2Parser.Set_varContext):
        pass

    # Exit a parse tree produced by Ah2Parser#set_var.
    def exitSet_var(self, ctx:Ah2Parser.Set_varContext):
        pass


    # Enter a parse tree produced by Ah2Parser#scoping.
    def enterScoping(self, ctx:Ah2Parser.ScopingContext):
        pass

    # Exit a parse tree produced by Ah2Parser#scoping.
    def exitScoping(self, ctx:Ah2Parser.ScopingContext):
        pass


    # Enter a parse tree produced by Ah2Parser#name.
    def enterName(self, ctx:Ah2Parser.NameContext):
        pass

    # Exit a parse tree produced by Ah2Parser#name.
    def exitName(self, ctx:Ah2Parser.NameContext):
        pass


    # Enter a parse tree produced by Ah2Parser#exports.
    def enterExports(self, ctx:Ah2Parser.ExportsContext):
        pass

    # Exit a parse tree produced by Ah2Parser#exports.
    def exitExports(self, ctx:Ah2Parser.ExportsContext):
        pass


    # Enter a parse tree produced by Ah2Parser#imports.
    def enterImports(self, ctx:Ah2Parser.ImportsContext):
        pass

    # Exit a parse tree produced by Ah2Parser#imports.
    def exitImports(self, ctx:Ah2Parser.ImportsContext):
        pass


    # Enter a parse tree produced by Ah2Parser#execute.
    def enterExecute(self, ctx:Ah2Parser.ExecuteContext):
        pass

    # Exit a parse tree produced by Ah2Parser#execute.
    def exitExecute(self, ctx:Ah2Parser.ExecuteContext):
        pass


    # Enter a parse tree produced by Ah2Parser#inject.
    def enterInject(self, ctx:Ah2Parser.InjectContext):
        pass

    # Exit a parse tree produced by Ah2Parser#inject.
    def exitInject(self, ctx:Ah2Parser.InjectContext):
        pass


    # Enter a parse tree produced by Ah2Parser#variable_types.
    def enterVariable_types(self, ctx:Ah2Parser.Variable_typesContext):
        pass

    # Exit a parse tree produced by Ah2Parser#variable_types.
    def exitVariable_types(self, ctx:Ah2Parser.Variable_typesContext):
        pass





# System import
import json
import shlex

# Library import
import click

# Application import
from ahRequests.request import Request
from httpPlug import HttpPlug
from headerBuilder import HeaderBuilder


# Handles custom commands
# This allows a user to access an arbitrary endpoint
class Custom(Request):
    def __init__(self, header, debug, sensitive):
        Request.__init__(self, '', header.get(), '', debug=debug, sensitive=sensitive)

    # Executes the passed command
    def handle(self, command):
        if (type(command) is not list):
            command = shlex.split(command.strip())
        if (len(command) < 1):
            return

        cType = None

        if ('--get' in command):
            cType = 'get'
        elif ('--post' in command):
            cType = 'post'
        elif ('--put' in command):
            cType = 'put'
        elif ('--patch' in command):
            cType = 'patch'
        elif ('--delete' in command):
            cType = 'delete'
        else:
            print('Custom requests require a type e.g. --get or --post')
            return

        if ('--url' not in command):
            print('Custom requests require a --url')
            return

        url = command[command.index('--url') + 1]

        postData = ''  # Post Data
        paramData = ''  # Data container in the URL, but only the part after the ?: someurl.com/user/7?thisIsAParamData=true
        pathData = ''  # Data contained in the URL
        headerData = ''  # Data contained in the headers

        if ('--data-post' in command):
            postData = command[command.index('--data-post') + 1]

        if ('--data-param' in command):
            paramData = command[command.index('--data-param') + 1]

        if ('--data-path' in command):
            pathData = command[command.index('--data-path') + 1]

        if ('--data-header' in command):
            headerData = command[command.index('--data-header') + 1]

        self.setUrl(url)
        # print(command)
        if (postData != ''):
            self.setPostData(json.loads(str(postData)))
        if (paramData != ''):
            self.setParamData(json.loads(str(paramData)))
        if (pathData != ''):
            self.setPathData(json.loads(str(pathData)))
        if (headerData != ''):
            header = HeaderBuilder()
            header.build(self.headers)
            header.build(json.loads(str(headerData)))
            self.setHeaders(header.get())

        if (cType == 'get'):
            self.get()
        elif (cType == 'post'):
            self.post()
        elif (cType == 'put'):
            self.put()
        elif (cType == 'patch'):
            self.patch()
        elif (cType == 'delete'):
            self.delete()
        else:
            print('Invalid custom request type')



# Library import
import click

# Application import
from custom import Custom
from httpPlug import HttpPlug
from headerBuilder import HeaderBuilder


# Handles creating an authentication request for tokenable drivers
# It does this by creating a command to an authentication endpoint with the applicable data
class AuthRequest(Custom):
    def __init__(self, username, password, http, debug, config):
        self.http = http

        header = HeaderBuilder()
        header.build(http.getContentTypeJSON())

        if (not http.isCredentialsPosted):
            header.build(http.getPasswordAuthHeader(username, password))

        Custom.__init__(self, header, debug, True)

        if (http.isCredentialsPosted):
            self.setPostData(self.http.getPasswordAuthData(username, password))

        if (self.debug):
            print('AuthRequest Created')

        self.custom = '--post --url ' + http.getAuthEndpoint(config) + ' --data-path \'{"user":"' + username + '"}\''

    def authenticate(self):
        self.handle(self.custom)

    def getToken(self):
        return self.http.getToken(self)  # request.headers.get('X-Subject-Token')



import base64
import json


# Base class for driver plugs
# Defines many customizable properties for interfacing to a new API type
class HttpPlug:

    def getAuthEndpoint(self, config):
        return config.get('auth-endpoint')

    def getPasswordAuthHeader(self, username, password):
        temp = username + ':' + password
        return {'Authorization': 'Basic ' + base64.b64encode(temp.encode('utf-8'))}

    def getPasswordAuthData(self, username, password):
        return {'username': username, 'password': password}

    def getTokenAuthHeader(self, token):
        return {'Authorization': 'Token token="' + token + '"'}

    def getContentTypeJSON(self):
        return {'Content-type': 'application/json'}

    def getToken(self, response):
        return json.loads(response.getResponseBody())['token']

    # Whether or not authentication can produce a usable token or
    # whether to use the username and password for each further request
    def isTokenable(self):
        return True

    # Whether or not crendetials or put into the post data or the header of the
    # authentication request
    def isCredentialsPosted(self):
        return False



# Generated from D:/Programming/Projects/NoPatience/API-Middleware/grammar/src\test1.g4 by ANTLR 4.7
# encoding: utf-8
from antlr4 import *
from io import StringIO
from typing import TextIO
import sys


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\5")
        buf.write("\t\4\2\t\2\3\2\3\2\3\2\3\2\3\2\2\2\3\2\2\2\2\7\2\4\3\2")
        buf.write("\2\2\4\5\7\4\2\2\5\6\7\3\2\2\6\7\7\4\2\2\7\3\3\2\2\2\2")
        return buf.getvalue()


class test1Parser(Parser):
    grammarFileName = "test1.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [DFA(ds, i) for i, ds in enumerate(atn.decisionToState)]

    sharedContextCache = PredictionContextCache()

    literalNames = ["<INVALID>", "'+'", "<INVALID>", "' '"]

    symbolicNames = ["<INVALID>", "<INVALID>", "NUMBER", "WHITESPACE"]

    RULE_operation = 0

    ruleNames = ["operation"]

    EOF = Token.EOF
    T__0 = 1
    NUMBER = 2
    WHITESPACE = 3

    def __init__(self, input: TokenStream, output: TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.7")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None

    class OperationContext(ParserRuleContext):

        def __init__(self, parser, parent: ParserRuleContext = None, invokingState: int = -1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NUMBER(self, i: int = None):
            if i is None:
                return self.getTokens(test1Parser.NUMBER)
            else:
                return self.getToken(test1Parser.NUMBER, i)

        def getRuleIndex(self):
            return test1Parser.RULE_operation

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterOperation"):
                listener.enterOperation(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitOperation"):
                listener.exitOperation(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitOperation"):
                return visitor.visitOperation(self)
            else:
                return visitor.visitChildren(self)

    def operation(self):

        localctx = test1Parser.OperationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_operation)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2
            self.match(test1Parser.NUMBER)
            self.state = 3
            self.match(test1Parser.T__0)
            self.state = 4
            self.match(test1Parser.NUMBER)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx



import sys
from antlr4 import *
from apitax.grammar.build.Ah2Parser import Ah2Parser
from apitax.grammar.build.Ah2Listener import Ah2Listener as Ah2ListenerOriginal


class Ah2Listener(Ah2ListenerOriginal):
    # Enter a parse tree produced by Ah2Parser#prog.
    def enterProg(self, ctx):
        print('prog entered')
        print(ctx)

    # Exit a parse tree produced by Ah2Parser#prog.
    def exitProg(self, ctx):
        print('prog left')
        print(ctx)

    # Enter a parse tree produced by Ah2Parser#statements.
    def enterStatements(self, ctx):
        print('statements entered')
        print(ctx)

    # Exit a parse tree produced by Ah2Parser#statements.
    def exitStatements(self, ctx):
        print('statements left')
        print(ctx)

    # Enter a parse tree produced by Ah2Parser#statement.
    def enterStatement(self, ctx):
        print('statement entered')
        print(ctx)

    # Exit a parse tree produced by Ah2Parser#statement.
    def exitStatement(self, ctx):
        print('statement left')
        print(ctx)

    # Enter a parse tree produced by Ah2Parser#r.
    def enterR(self, ctx):  #:Ah2Parser.RContext):
        print('r entered')
        print(ctx.INT())

    # Exit a parse tree produced by Ah2Parser#r.
    def exitR(self, ctx):
        print('r left')
        print(ctx)



import sys
from antlr4 import *
from apitax.grammar.build.Ah2Lexer import Ah2Lexer
from apitax.grammar.build.Ah2Parser import Ah2Parser
from apitax.grammar.Ah2Listener import Ah2Listener
from apitax.grammar.Ah2Visitor import Ah2Visitor


def GrammarTest(filepath):
    # input = FileStream(argv[1])
    input = FileStream(filepath)
    lexer = Ah2Lexer(input)
    stream = CommonTokenStream(lexer)
    parser = Ah2Parser(stream)
    tree = parser.prog()
    printer = Ah2Listener()

    visitor = Ah2Visitor()
    result = visitor.visit(tree)
    # print(result)

    # walker = ParseTreeWalker()
    # walker.walk(printer, tree)


if __name__ == '__main__':
    GrammarTest(sys.argv)



import configparser


# Reads the config and allows us to retrieve values from it
class Config:
    def __init__(self):
        self.cp = configparser.RawConfigParser()
        cFilePath = 'config.txt'
        self.cp.read(cFilePath)

    def get(self, param):
        return self.cp.get('Config', param)

    def has(self, param):
        return self.cp.has_option('Config', param)



import sys


class SystemCommands:

    def __init__(self):
        self.args = sys.argv[1:]

    def isEmpty(self):
        if(len(self.args) < 1):
            return True
        return False

    def isArg(self, arg):
        if (arg in self.args):
            return True
        return False

    def getArg(self, arg, default=""):
        if (not self.isArg(arg)):
            if default != "":
                return default
            else:
                return False
        return self.args[self.args.index(arg) + 1]



# Import Drivers below here

from drivers import *


# End Driver Imports


# Factory class for creating HttpPlug Drivers
class HttpPlugFactory:

    @staticmethod
    def make(name):
        # import_submodules(drivers)
        constructor = globals()[name]
        return constructor()



# Import Drivers
from ahRequests.drivers import *


# Used to create driver command instances
class DriverCommandsFactory:

    @staticmethod
    def make(name):
        if (name in globals()):
            constructor = globals()[name]
            return constructor()
        return False



# Generated from D:/Programming/Projects/Apitax/grammar/src\Ah2.g4 by ANTLR 4.7
# encoding: utf-8
from antlr4 import *
from io import StringIO
from typing.io import TextIO
import sys

def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\60")
        buf.write("g\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7\4\b")
        buf.write("\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\3\2\3\2")
        buf.write("\3\2\3\3\6\3\37\n\3\r\3\16\3 \3\4\3\4\3\4\3\4\3\4\3\4")
        buf.write("\5\4)\n\4\3\5\3\5\3\5\3\5\3\5\3\5\3\5\5\5\62\n\5\3\5\3")
        buf.write("\5\3\5\3\5\3\5\3\5\3\5\3\5\3\5\7\5=\n\5\f\5\16\5@\13\5")
        buf.write("\3\6\3\6\3\6\3\6\3\6\3\6\5\6H\n\6\3\7\3\7\3\7\5\7M\n\7")
        buf.write("\3\b\3\b\3\b\5\bR\n\b\3\t\3\t\3\t\5\tW\n\t\3\n\3\n\3\n")
        buf.write("\3\13\3\13\3\f\3\f\5\f`\n\f\3\f\3\f\3\f\3\r\3\r\3\r\2")
        buf.write("\3\b\16\2\4\6\b\n\f\16\20\22\24\26\30\2\6\3\2\5\6\3\2")
        buf.write("\3\4\3\2*+\4\2\23\24..\2l\2\32\3\2\2\2\4\36\3\2\2\2\6")
        buf.write("(\3\2\2\2\b\61\3\2\2\2\nA\3\2\2\2\fL\3\2\2\2\16N\3\2\2")
        buf.write("\2\20S\3\2\2\2\22X\3\2\2\2\24[\3\2\2\2\26]\3\2\2\2\30")
        buf.write("d\3\2\2\2\32\33\5\4\3\2\33\34\7\2\2\3\34\3\3\2\2\2\35")
        buf.write("\37\5\6\4\2\36\35\3\2\2\2\37 \3\2\2\2 \36\3\2\2\2 !\3")
        buf.write("\2\2\2!\5\3\2\2\2\")\5\26\f\2#)\5\24\13\2$)\5\b\5\2%)")
        buf.write("\5\n\6\2&)\5\f\7\2\')\7/\2\2(\"\3\2\2\2(#\3\2\2\2($\3")
        buf.write("\2\2\2(%\3\2\2\2(&\3\2\2\2(\'\3\2\2\2)\7\3\2\2\2*+\b\5")
        buf.write("\1\2+\62\5\30\r\2,\62\5\26\f\2-.\7\35\2\2./\5\b\5\2/\60")
        buf.write("\7\36\2\2\60\62\3\2\2\2\61*\3\2\2\2\61,\3\2\2\2\61-\3")
        buf.write("\2\2\2\62>\3\2\2\2\63\64\f\5\2\2\64\65\7\7\2\2\65=\5\b")
        buf.write("\5\5\66\67\f\4\2\2\678\t\2\2\28=\5\b\5\59:\f\3\2\2:;\t")
        buf.write("\3\2\2;=\5\b\5\4<\63\3\2\2\2<\66\3\2\2\2<9\3\2\2\2=@\3")
        buf.write("\2\2\2><\3\2\2\2>?\3\2\2\2?\t\3\2\2\2@>\3\2\2\2AB\7$\2")
        buf.write("\2BC\7*\2\2CG\7\b\2\2DH\5\b\5\2EH\5\30\r\2FH\5\24\13\2")
        buf.write("GD\3\2\2\2GE\3\2\2\2GF\3\2\2\2H\13\3\2\2\2IM\5\22\n\2")
        buf.write("JM\5\20\t\2KM\5\16\b\2LI\3\2\2\2LJ\3\2\2\2LK\3\2\2\2M")
        buf.write("\r\3\2\2\2NQ\7!\2\2OR\7*\2\2PR\5\26\f\2QO\3\2\2\2QP\3")
        buf.write("\2\2\2R\17\3\2\2\2SV\7#\2\2TW\7+\2\2UW\5\24\13\2VT\3\2")
        buf.write("\2\2VU\3\2\2\2W\21\3\2\2\2XY\7\"\2\2YZ\5\24\13\2Z\23\3")
        buf.write("\2\2\2[\\\7-\2\2\\\25\3\2\2\2]_\7\33\2\2^`\7)\2\2_^\3")
        buf.write("\2\2\2_`\3\2\2\2`a\3\2\2\2ab\t\4\2\2bc\7\34\2\2c\27\3")
        buf.write("\2\2\2de\t\5\2\2e\31\3\2\2\2\f (\61<>GLQV_")
        return buf.getvalue()


class Ah2Parser ( Parser ):

    grammarFileName = "Ah2.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'+'", "'-'", "'*'", "'/'", "'^'", "'='", 
                     "'>'", "'<'", "'>='", "'<='", "'!='", "'!'", "'_'", 
                     "'.'", "':'", "'%'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "'{%'", "'%}'", "'{'", "'}'", "'{{'", 
                     "'}}'", "'('", "')'" ]

    symbolicNames = [ "<INVALID>", "PLUS", "MINUS", "MUL", "DIV", "POW", 
                      "EQUAL", "GT", "LT", "GE", "LE", "NE", "NOT", "ULINE", 
                      "DOT", "COLON", "PERCENT", "BOOLEAN", "NUMBER", "INT", 
                      "FLOAT", "EXECUTEOPEN", "EXECUTECLOSE", "BLOCKOPEN", 
                      "BLOCKCLOSE", "MUSTACHEOPEN", "MUSTACHECLOSE", "LPAREN", 
                      "RPAREN", "FALSE", "TRUE", "NAME", "IMPORT", "EXPORT", 
                      "SET", "IF", "THEN", "ELSE", "ENDIF", "REQUEST", "LABEL", 
                      "DOT_LABEL", "HEX", "COMMANDTAX", "STRING", "NEWLINE", 
                      "WS" ]

    RULE_prog = 0
    RULE_statements = 1
    RULE_statement = 2
    RULE_expr = 3
    RULE_set_var = 4
    RULE_scoping = 5
    RULE_name = 6
    RULE_exports = 7
    RULE_imports = 8
    RULE_execute = 9
    RULE_inject = 10
    RULE_variable_types = 11

    ruleNames =  [ "prog", "statements", "statement", "expr", "set_var", 
                   "scoping", "name", "exports", "imports", "execute", "inject", 
                   "variable_types" ]

    EOF = Token.EOF
    PLUS=1
    MINUS=2
    MUL=3
    DIV=4
    POW=5
    EQUAL=6
    GT=7
    LT=8
    GE=9
    LE=10
    NE=11
    NOT=12
    ULINE=13
    DOT=14
    COLON=15
    PERCENT=16
    BOOLEAN=17
    NUMBER=18
    INT=19
    FLOAT=20
    EXECUTEOPEN=21
    EXECUTECLOSE=22
    BLOCKOPEN=23
    BLOCKCLOSE=24
    MUSTACHEOPEN=25
    MUSTACHECLOSE=26
    LPAREN=27
    RPAREN=28
    FALSE=29
    TRUE=30
    NAME=31
    IMPORT=32
    EXPORT=33
    SET=34
    IF=35
    THEN=36
    ELSE=37
    ENDIF=38
    REQUEST=39
    LABEL=40
    DOT_LABEL=41
    HEX=42
    COMMANDTAX=43
    STRING=44
    NEWLINE=45
    WS=46

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.7")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None



    class ProgContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def statements(self):
            return self.getTypedRuleContext(Ah2Parser.StatementsContext,0)


        def EOF(self):
            return self.getToken(Ah2Parser.EOF, 0)

        def getRuleIndex(self):
            return Ah2Parser.RULE_prog

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProg" ):
                listener.enterProg(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProg" ):
                listener.exitProg(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProg" ):
                return visitor.visitProg(self)
            else:
                return visitor.visitChildren(self)




    def prog(self):

        localctx = Ah2Parser.ProgContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_prog)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 24
            self.statements()
            self.state = 25
            self.match(Ah2Parser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class StatementsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(Ah2Parser.StatementContext)
            else:
                return self.getTypedRuleContext(Ah2Parser.StatementContext,i)


        def getRuleIndex(self):
            return Ah2Parser.RULE_statements

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStatements" ):
                listener.enterStatements(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStatements" ):
                listener.exitStatements(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStatements" ):
                return visitor.visitStatements(self)
            else:
                return visitor.visitChildren(self)




    def statements(self):

        localctx = Ah2Parser.StatementsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_statements)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 28 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 27
                self.statement()
                self.state = 30 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << Ah2Parser.BOOLEAN) | (1 << Ah2Parser.NUMBER) | (1 << Ah2Parser.MUSTACHEOPEN) | (1 << Ah2Parser.LPAREN) | (1 << Ah2Parser.NAME) | (1 << Ah2Parser.IMPORT) | (1 << Ah2Parser.EXPORT) | (1 << Ah2Parser.SET) | (1 << Ah2Parser.COMMANDTAX) | (1 << Ah2Parser.STRING) | (1 << Ah2Parser.NEWLINE))) != 0)):
                    break

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class StatementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def inject(self):
            return self.getTypedRuleContext(Ah2Parser.InjectContext,0)


        def execute(self):
            return self.getTypedRuleContext(Ah2Parser.ExecuteContext,0)


        def expr(self):
            return self.getTypedRuleContext(Ah2Parser.ExprContext,0)


        def set_var(self):
            return self.getTypedRuleContext(Ah2Parser.Set_varContext,0)


        def scoping(self):
            return self.getTypedRuleContext(Ah2Parser.ScopingContext,0)


        def NEWLINE(self):
            return self.getToken(Ah2Parser.NEWLINE, 0)

        def getRuleIndex(self):
            return Ah2Parser.RULE_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStatement" ):
                listener.enterStatement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStatement" ):
                listener.exitStatement(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStatement" ):
                return visitor.visitStatement(self)
            else:
                return visitor.visitChildren(self)




    def statement(self):

        localctx = Ah2Parser.StatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_statement)
        try:
            self.state = 38
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,1,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 32
                self.inject()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 33
                self.execute()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 34
                self.expr(0)
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 35
                self.set_var()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 36
                self.scoping()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 37
                self.match(Ah2Parser.NEWLINE)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ExprContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def variable_types(self):
            return self.getTypedRuleContext(Ah2Parser.Variable_typesContext,0)


        def inject(self):
            return self.getTypedRuleContext(Ah2Parser.InjectContext,0)


        def LPAREN(self):
            return self.getToken(Ah2Parser.LPAREN, 0)

        def expr(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(Ah2Parser.ExprContext)
            else:
                return self.getTypedRuleContext(Ah2Parser.ExprContext,i)


        def RPAREN(self):
            return self.getToken(Ah2Parser.RPAREN, 0)

        def POW(self):
            return self.getToken(Ah2Parser.POW, 0)

        def MUL(self):
            return self.getToken(Ah2Parser.MUL, 0)

        def DIV(self):
            return self.getToken(Ah2Parser.DIV, 0)

        def PLUS(self):
            return self.getToken(Ah2Parser.PLUS, 0)

        def MINUS(self):
            return self.getToken(Ah2Parser.MINUS, 0)

        def getRuleIndex(self):
            return Ah2Parser.RULE_expr

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExpr" ):
                listener.enterExpr(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExpr" ):
                listener.exitExpr(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExpr" ):
                return visitor.visitExpr(self)
            else:
                return visitor.visitChildren(self)



    def expr(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = Ah2Parser.ExprContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 6
        self.enterRecursionRule(localctx, 6, self.RULE_expr, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 47
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [Ah2Parser.BOOLEAN, Ah2Parser.NUMBER, Ah2Parser.STRING]:
                self.state = 41
                self.variable_types()
                pass
            elif token in [Ah2Parser.MUSTACHEOPEN]:
                self.state = 42
                self.inject()
                pass
            elif token in [Ah2Parser.LPAREN]:
                self.state = 43
                self.match(Ah2Parser.LPAREN)
                self.state = 44
                self.expr(0)
                self.state = 45
                self.match(Ah2Parser.RPAREN)
                pass
            else:
                raise NoViableAltException(self)

            self._ctx.stop = self._input.LT(-1)
            self.state = 60
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,4,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    self.state = 58
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,3,self._ctx)
                    if la_ == 1:
                        localctx = Ah2Parser.ExprContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_expr)
                        self.state = 49
                        if not self.precpred(self._ctx, 3):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 3)")
                        self.state = 50
                        self.match(Ah2Parser.POW)
                        self.state = 51
                        self.expr(3)
                        pass

                    elif la_ == 2:
                        localctx = Ah2Parser.ExprContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_expr)
                        self.state = 52
                        if not self.precpred(self._ctx, 2):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                        self.state = 53
                        _la = self._input.LA(1)
                        if not(_la==Ah2Parser.MUL or _la==Ah2Parser.DIV):
                            self._errHandler.recoverInline(self)
                        else:
                            self._errHandler.reportMatch(self)
                            self.consume()
                        self.state = 54
                        self.expr(3)
                        pass

                    elif la_ == 3:
                        localctx = Ah2Parser.ExprContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_expr)
                        self.state = 55
                        if not self.precpred(self._ctx, 1):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 1)")
                        self.state = 56
                        _la = self._input.LA(1)
                        if not(_la==Ah2Parser.PLUS or _la==Ah2Parser.MINUS):
                            self._errHandler.recoverInline(self)
                        else:
                            self._errHandler.reportMatch(self)
                            self.consume()
                        self.state = 57
                        self.expr(2)
                        pass

             
                self.state = 62
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,4,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx

    class Set_varContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def SET(self):
            return self.getToken(Ah2Parser.SET, 0)

        def LABEL(self):
            return self.getToken(Ah2Parser.LABEL, 0)

        def EQUAL(self):
            return self.getToken(Ah2Parser.EQUAL, 0)

        def expr(self):
            return self.getTypedRuleContext(Ah2Parser.ExprContext,0)


        def variable_types(self):
            return self.getTypedRuleContext(Ah2Parser.Variable_typesContext,0)


        def execute(self):
            return self.getTypedRuleContext(Ah2Parser.ExecuteContext,0)


        def getRuleIndex(self):
            return Ah2Parser.RULE_set_var

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSet_var" ):
                listener.enterSet_var(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSet_var" ):
                listener.exitSet_var(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSet_var" ):
                return visitor.visitSet_var(self)
            else:
                return visitor.visitChildren(self)




    def set_var(self):

        localctx = Ah2Parser.Set_varContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_set_var)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 63
            self.match(Ah2Parser.SET)
            self.state = 64
            self.match(Ah2Parser.LABEL)
            self.state = 65
            self.match(Ah2Parser.EQUAL)
            self.state = 69
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,5,self._ctx)
            if la_ == 1:
                self.state = 66
                self.expr(0)
                pass

            elif la_ == 2:
                self.state = 67
                self.variable_types()
                pass

            elif la_ == 3:
                self.state = 68
                self.execute()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ScopingContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def imports(self):
            return self.getTypedRuleContext(Ah2Parser.ImportsContext,0)


        def exports(self):
            return self.getTypedRuleContext(Ah2Parser.ExportsContext,0)


        def name(self):
            return self.getTypedRuleContext(Ah2Parser.NameContext,0)


        def getRuleIndex(self):
            return Ah2Parser.RULE_scoping

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterScoping" ):
                listener.enterScoping(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitScoping" ):
                listener.exitScoping(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitScoping" ):
                return visitor.visitScoping(self)
            else:
                return visitor.visitChildren(self)




    def scoping(self):

        localctx = Ah2Parser.ScopingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_scoping)
        try:
            self.state = 74
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [Ah2Parser.IMPORT]:
                self.enterOuterAlt(localctx, 1)
                self.state = 71
                self.imports()
                pass
            elif token in [Ah2Parser.EXPORT]:
                self.enterOuterAlt(localctx, 2)
                self.state = 72
                self.exports()
                pass
            elif token in [Ah2Parser.NAME]:
                self.enterOuterAlt(localctx, 3)
                self.state = 73
                self.name()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class NameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(Ah2Parser.NAME, 0)

        def LABEL(self):
            return self.getToken(Ah2Parser.LABEL, 0)

        def inject(self):
            return self.getTypedRuleContext(Ah2Parser.InjectContext,0)


        def getRuleIndex(self):
            return Ah2Parser.RULE_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterName" ):
                listener.enterName(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitName" ):
                listener.exitName(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitName" ):
                return visitor.visitName(self)
            else:
                return visitor.visitChildren(self)




    def name(self):

        localctx = Ah2Parser.NameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 76
            self.match(Ah2Parser.NAME)
            self.state = 79
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [Ah2Parser.LABEL]:
                self.state = 77
                self.match(Ah2Parser.LABEL)
                pass
            elif token in [Ah2Parser.MUSTACHEOPEN]:
                self.state = 78
                self.inject()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ExportsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EXPORT(self):
            return self.getToken(Ah2Parser.EXPORT, 0)

        def DOT_LABEL(self):
            return self.getToken(Ah2Parser.DOT_LABEL, 0)

        def execute(self):
            return self.getTypedRuleContext(Ah2Parser.ExecuteContext,0)


        def getRuleIndex(self):
            return Ah2Parser.RULE_exports

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExports" ):
                listener.enterExports(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExports" ):
                listener.exitExports(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExports" ):
                return visitor.visitExports(self)
            else:
                return visitor.visitChildren(self)




    def exports(self):

        localctx = Ah2Parser.ExportsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_exports)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 81
            self.match(Ah2Parser.EXPORT)
            self.state = 84
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [Ah2Parser.DOT_LABEL]:
                self.state = 82
                self.match(Ah2Parser.DOT_LABEL)
                pass
            elif token in [Ah2Parser.COMMANDTAX]:
                self.state = 83
                self.execute()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ImportsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IMPORT(self):
            return self.getToken(Ah2Parser.IMPORT, 0)

        def execute(self):
            return self.getTypedRuleContext(Ah2Parser.ExecuteContext,0)


        def getRuleIndex(self):
            return Ah2Parser.RULE_imports

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterImports" ):
                listener.enterImports(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitImports" ):
                listener.exitImports(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitImports" ):
                return visitor.visitImports(self)
            else:
                return visitor.visitChildren(self)




    def imports(self):

        localctx = Ah2Parser.ImportsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_imports)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 86
            self.match(Ah2Parser.IMPORT)
            self.state = 87
            self.execute()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ExecuteContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMMANDTAX(self):
            return self.getToken(Ah2Parser.COMMANDTAX, 0)

        def getRuleIndex(self):
            return Ah2Parser.RULE_execute

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExecute" ):
                listener.enterExecute(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExecute" ):
                listener.exitExecute(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExecute" ):
                return visitor.visitExecute(self)
            else:
                return visitor.visitChildren(self)




    def execute(self):

        localctx = Ah2Parser.ExecuteContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_execute)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 89
            self.match(Ah2Parser.COMMANDTAX)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class InjectContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def MUSTACHEOPEN(self):
            return self.getToken(Ah2Parser.MUSTACHEOPEN, 0)

        def MUSTACHECLOSE(self):
            return self.getToken(Ah2Parser.MUSTACHECLOSE, 0)

        def DOT_LABEL(self):
            return self.getToken(Ah2Parser.DOT_LABEL, 0)

        def LABEL(self):
            return self.getToken(Ah2Parser.LABEL, 0)

        def REQUEST(self):
            return self.getToken(Ah2Parser.REQUEST, 0)

        def getRuleIndex(self):
            return Ah2Parser.RULE_inject

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInject" ):
                listener.enterInject(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInject" ):
                listener.exitInject(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInject" ):
                return visitor.visitInject(self)
            else:
                return visitor.visitChildren(self)




    def inject(self):

        localctx = Ah2Parser.InjectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_inject)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 91
            self.match(Ah2Parser.MUSTACHEOPEN)
            self.state = 93
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==Ah2Parser.REQUEST:
                self.state = 92
                self.match(Ah2Parser.REQUEST)


            self.state = 95
            _la = self._input.LA(1)
            if not(_la==Ah2Parser.LABEL or _la==Ah2Parser.DOT_LABEL):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 96
            self.match(Ah2Parser.MUSTACHECLOSE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Variable_typesContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def BOOLEAN(self):
            return self.getToken(Ah2Parser.BOOLEAN, 0)

        def NUMBER(self):
            return self.getToken(Ah2Parser.NUMBER, 0)

        def STRING(self):
            return self.getToken(Ah2Parser.STRING, 0)

        def getRuleIndex(self):
            return Ah2Parser.RULE_variable_types

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterVariable_types" ):
                listener.enterVariable_types(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitVariable_types" ):
                listener.exitVariable_types(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitVariable_types" ):
                return visitor.visitVariable_types(self)
            else:
                return visitor.visitChildren(self)




    def variable_types(self):

        localctx = Ah2Parser.Variable_typesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_variable_types)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 98
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << Ah2Parser.BOOLEAN) | (1 << Ah2Parser.NUMBER) | (1 << Ah2Parser.STRING))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx



    def sempred(self, localctx:RuleContext, ruleIndex:int, predIndex:int):
        if self._predicates == None:
            self._predicates = dict()
        self._predicates[3] = self.expr_sempred
        pred = self._predicates.get(ruleIndex, None)
        if pred is None:
            raise Exception("No predicate with index:" + str(ruleIndex))
        else:
            return pred(localctx, predIndex)

    def expr_sempred(self, localctx:ExprContext, predIndex:int):
            if predIndex == 0:
                return self.precpred(self._ctx, 3)
         

            if predIndex == 1:
                return self.precpred(self._ctx, 2)
         

            if predIndex == 2:
                return self.precpred(self._ctx, 1)
         







from apitax.grammar.build.Ah2Parser import Ah2Parser
from apitax.grammar.build.Ah2Visitor import Ah2Visitor as Ah2VisitorOriginal
from apitax.ah.scriptax.ScriptData import ScriptData as DataStore
from apitax.logs.Log import Log
import json

class Ah2Visitor(Ah2VisitorOriginal):

    def __init__(self, debug = True):
        self.data = DataStore()
        self.log = Log('logs/log.log')
        self.debug = debug

    # Visit a parse tree produced by Ah2Parser#prog.
    def visitProg(self, ctx):
        temp = self.visitChildren(ctx)
        # print('proj: '+str(temp))
        return temp

    # Visit a parse tree produced by Ah2Parser#statements.
    def visitStatements(self, ctx):
        temp = self.visitChildren(ctx)
        # print('result: '+str(temp))
        return temp

    # Visit a parse tree produced by Ah2Parser#statement.
    def visitStatement(self, ctx):
        line = ctx.getText().strip()
        if(line != ""):
            self.log.log('> Now processing: ' + line)
            self.log.log('')

        temp = self.visitChildren(ctx)
        if (ctx.expr() is not None):
            if (self.debug):
                self.log.log('> Evaluated Expression: ' + str(temp))
                self.log.log('')

        if (line != ""):
            self.log.log('')

        return temp

    # Visit a parse tree produced by Ah2Parser#expr.
    def visitExpr(self, ctx):  # Get number of terms and loop this code for #terms - 1
        if (ctx.variable_types()):
            return self.visit(ctx.variable_types())

        if (ctx.LPAREN()):
            return self.visit(ctx.expr(0))

        if(ctx.inject()):
            return self.visit(ctx.inject())

        if (ctx.POW()):
            return self.visit(ctx.expr(0)) ** self.visit(ctx.expr(1))

        if (ctx.PLUS()):
            return self.visit(ctx.expr(0)) + self.visit(ctx.expr(1))

        if (ctx.MINUS()):
            return self.visit(ctx.expr(0)) - self.visit(ctx.expr(1))

        if (ctx.MUL()):
            return self.visit(ctx.expr(0)) * self.visit(ctx.expr(1))

        if (ctx.DIV()):
            return self.visit(ctx.expr(0)) / self.visit(ctx.expr(1))

        return self.visitChildren(ctx)

    # Visit a parse tree produced by Ah2Parser#set_var.
    def visitSet_var(self, ctx):

        label = ctx.LABEL().getText()

        if (ctx.variable_types()):
            self.data.storeVar(label, self.visit(ctx.variable_types()))

        if (ctx.execute()):
            self.data.storeVar(label, self.visit(ctx.execute()))
            # self.variables[ctx.LABEL().getText()] = ctx.COMMANDTAX().getText()

        if(ctx.expr()):
            self.data.storeVar(label, self.visit(ctx.expr()))

        if (self.debug):
            self.log.log('> Assigning Variable: ' + label + ' = ' + str(self.data.getVar(label)))
            self.log.log('')

        # print('set var:')
        # print(self.data.dataStore)

    # Visit a parse tree produced by Ah2Parser#scoping.
    def visitScoping(self, ctx):
        # print('scoping')
        return self.visitChildren(ctx)

    # Visit a parse tree produced by Ah2Parser#name.
    def visitName(self, ctx):
        if ctx.LABEL() is not None:
            self.data.name = ctx.LABEL().getText()

        if ctx.inject() is not None:
            self.data.name = self.visit(ctx.inject())

        if (self.debug):
            self.log.log('> Setting Script Name: ' + self.data.name)
            self.log.log('')

    # Visit a parse tree produced by Ah2Parser#exports.
    def visitExports(self, ctx):
        print('exports')
        return self.visitChildren(ctx)

    # Visit a parse tree produced by Ah2Parser#imports.
    def visitImports(self, ctx):
        print('imports')
        return self.visitChildren(ctx)

    # Visit a parse tree produced by Ah2Parser#execute.
    def visitExecute(self, ctx):
        print('execute')
        return self.visitChildren(ctx)

    # Visit a parse tree produced by Ah2Parser#inject.
    def visitInject(self, ctx: Ah2Parser.InjectContext):

        label = ""

        if(ctx.DOT_LABEL()):
            label = ctx.DOT_LABEL().getText()

        if(ctx.LABEL()):
            label = ctx.LABEL().getText()

        isRequest = ctx.REQUEST()
        returner = None

        try:
            if (isRequest):
                returner = self.data.getRequest(label)
            else:
                returner = self.data.getVar(label)
        except:
            self.log.error('Injection Failure. Cannot access variable: ' + json.dumps(label) + ' - Does it exist?')
            self.log.log('')
            return None

        if (self.debug):
            self.log.log('> Injecting Variable \'' + label + '\': ' + str(returner))
            self.log.log('')

        return returner


    # Visit a parse tree produced by Ah2Parser#variable.
    def visitVariable_types(self, ctx: Ah2Parser.Variable_typesContext):

        #print('visit variable')

        if (ctx.NUMBER() is not None):
            return float(ctx.NUMBER().getText())

        if (ctx.STRING() is not None):
            return ctx.STRING().getText()[1:-1]

        if (ctx.BOOLEAN() is not None):
            return str(ctx.BOOLEAN().getText()).lower() == "true"

        #print('Impossible Variable')

        # return self.visitChildren(ctx)



# Builder class for creating headers dynamically
class HeaderBuilder:
    def __init__(self):
        self.header = {}

    def build(self, header):
        self.header.update(header)

    def remove(self, header):
        return self.header.pop(header, None)

    def get(self):
        return self.header



# System import
import json
import re
import os

# Application import
from log import Log
from ahRequests.request import Request
from ahRequests.scriptData import ScriptData


# TODO:
#  Change the way request is saved to include status code & command run
#  Add additional syntaxes for import & exporting


# Script is used to automate the execution of many commands
class Script(Request):
    def __init__(self, header, debug, sensitive):
        self.header = header
        self.debug = debug
        self.sensitive = sensitive
        self.config = None
        self.data = ScriptData()
        self.log = Log('logs/log.log')

        self.regexSetVar = '^set[ ]{1,}[A-z0-9_.\-]{2,}[ ]{0,}=[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}_ ]{1,}[ ]{0,}$'  # {{ someVarName = someVarValue }}
        self.regexSetUrl = '^{{[ ]{0,}url:[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}[\] _"\']{1,}[ ]{0,}}}'  # {{url: {"url":"https://stackoverflow.com/questions/4426663/{id.test}/how-to-remove-the-first-item-from-a-list", "name":"auth"}}}

        self.regexExportVar = '^export[ ]{1,}[A-z0-9_.\-]{1,}[ ]{0,}$'  # export someVar
        self.regexExportNewVar = '^export[ ]{1,}[A-z0-9_.\-]{2,}[ ]{0,}=[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}_]{1,}[ ]{0,}$'  # export user_id = {{user_id}}
        self.regexExportRequest = '^export[ ]{1,}[A-z0-9:/&$#@%?<>.,-{}_ \'"]{2,}[ ]{0,}$'

        self.regexImport = '^import[ ]{1,}[A-z0-9:/&$#@%?<>.,-{}_ \'"]{2,}[ ]{0,}$'

        self.regexName = '^name[ ]{1,}[A-z0-9_]{1,}[ ]{0,}$'  # name OpenStackURLs5

        self.regexExecuteCommandtax = '^{%[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}[\] _"\']{1,}[ ]{0,}%}$'  # {% some command tax %}
        self.regexCommandtax = '^>[ ]{1,}[A-z0-9:/&$#@%?<>.,\-{}[\] _"\']{1,}[ ]{0,}$'  # {% some command tax %}

        self.regexVar = '{{[ ]{0,}[A-z0-9_.\-]{1,}[ ]{0,}}}'
        self.regexUrl = '{{[ ]{0,}u[ ]{0,}:[ ]{0,}[A-z0-9_.]{1,}[ ]{0,}}}'
        self.regexRequest = '{{[ ]{0,}r[ ]{0,}:[ ]{0,}[A-z0-9_.\-]{1,}(?:[.][A-z0-9_.\-]{1,}){1,}[ ]{0,}}}'

        Request.__init__(self, '', header.get(), '', debug=debug, sensitive=sensitive, customResponse=True)

    def setConfig(self, config):
        self.config = config

    def detectScriptax(self, command, lineNumber=0):
        detected = False
        regexVar = '^{{[ ]{0,}[A-z0-9_.\-]{2,}[ ]{0,}=[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}_]{1,}[ ]{0,}}}'  # {{ someVarName = someVarValue }}
        regexUrl = '^{{[ ]{0,}url:[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}[\] _"\']{1,}[ ]{0,}}}'  # {{url: {"url":"https://stackoverflow.com/questions/4426663/{id.test}/how-to-remove-the-first-item-from-a-list", "name":"auth"}}}

        regexExportVar = '^export[ ]{1,}[A-z0-9_.\-]{1,}[ ]{0,}$'  # export someVar
        regexExportNewVar = '^export[ ]{1,}[A-z0-9_.\-]{2,}[ ]{0,}=[ ]{0,}[A-z0-9:/&$#@%?<>.,\-{}_]{1,}[ ]{0,}$'  # export user_id = {{user_id}}

        regexImport = '^import[ ]{1,}[A-z0-9:/&$#@%?<>.,-{}_ \'"]{2,}[ ]{0,}$'

        regexName = '^name[ ]{1,}[A-z0-9_]{1,}[ ]{0,}$'  # name OpenStackURLs5

        # Variable
        matches = re.findall(regexVar, command)
        for match in matches:
            detected = True
            matchStr = match[2:-2]
            var = matchStr.split('=')
            vName = var[0].strip()
            vVal = var[1].strip()
            self.data.storeVar(vName, vVal)
            if (self.debug):
                self.log.log('> Assigning Variable: ' + vName + ' = ' + vVal)
                self.log.log('')

        # URL
        matches = re.findall(regexUrl, command)
        for match in matches:
            detected = True
            matchStr = match[2:-2]
            urlData = json.loads(matchStr[matchStr.find(':') + 1:].strip())
            uName = urlData['name'].strip()
            uVal = urlData['url'].strip()
            self.data.storeUrl(uName, uVal)
            if (self.debug):
                self.log.log('> Assigning URL: ' + uName + ' = ' + uVal)
                self.log.log('')

        # Export Existing Variable
        matches = re.findall(regexExportVar, command)
        for match in matches:
            detected = True
            matchStr = (match[6:])
            varName = matchStr.strip()
            self.data.export(varName)
            if (self.debug):
                self.log.log('> Exporting Variable: ' + varName + ' = ' + self.data.getVar(varName))
                self.log.log('')

        # Export New Variable
        matches = re.findall(regexExportNewVar, command)
        for match in matches:
            detected = True
            matchStr = (match[6:]).strip()
            var = matchStr.split('=')
            vName = var[0].strip()
            vVal = var[1].strip()
            self.data.storeVar(vName, vVal)
            self.data.export(vName)
            if (self.debug):
                self.log.log('> Exporting New Variable: ' + vName + ' = ' + vVal)
                self.log.log('')

        # Import
        matches = re.findall(regexImport, command)
        for match in matches:
            from command import Command
            detected = True
            matchStr = (match[6:])
            importCommand = matchStr.strip()
            commandHandler = Command(self.header, importCommand, self.config, debug=self.debug,
                                     sensitive=self.sensitive)
            if (commandHandler.getRequest().getResponseBody().strip() != ''):
                if (isinstance(commandHandler.getRequest(), Script)):
                    if (commandHandler.getRequest().data.name == ""):
                        fileName = os.path.basename(importCommand[6:].strip())
                        if ("." in fileName):
                            fileName = fileName.split('.')[0]
                        commandHandler.getRequest().data.name = fileName
                    scriptName = commandHandler.getRequest().data.name

                    # self.data.store(commandHandler.getRequest().data.getRequestData(), prefix=scriptName)
                    self.data.importExports(commandHandler.getRequest().data.getExports(), prefix=scriptName)
                    # print(json.dumps(self.data.dataStore))
                else:
                    self.data.store(commandHandler.getRequest().getResponseBody())
                    # self.data.store(commandHandler.getRequest().getResponseBody())
                self.request['text'][lineNumber] = {"command": importCommand.strip(),
                                                    "status": commandHandler.getRequest().getResponseStatusCode(),
                                                    "body": json.loads(commandHandler.getRequest().getResponseBody())}
            else:
                self.request['text'][lineNumber] = {"command": importCommand.strip(),
                                                    "status": commandHandler.getRequest().getResponseStatusCode(),
                                                    "body": ""}
            if (self.debug):
                self.log.log('> Importing: ' + importCommand)
                self.log.log('')

        # Name
        matches = re.findall(regexName, command)
        for match in matches:
            detected = True
            matchStr = (match[4:])
            name = matchStr.strip()
            self.data.name = name
            if (self.debug):
                self.log.log('> Setting Script Name: ' + name)
                self.log.log('')

        if (detected and self.debug):
            self.log.log('')
            self.log.log('')

        return detected

    def inject(self, command):

        # regexVar = '{{[ ]{0,}v[ ]{0,}:[ ]{0,}[A-z0-9]{1,}[ ]{0,}}}'
        regexVar = '{{[ ]{0,}[A-z0-9_.\-]{1,}[ ]{0,}}}'
        regexUrl = '{{[ ]{0,}u[ ]{0,}:[ ]{0,}[A-z0-9_.]{1,}[ ]{0,}}}'
        regexRequest = '{{[ ]{0,}r[ ]{0,}:[ ]{0,}[0-9]{1,}(?:[.][A-z0-9]{1,}){1,}[ ]{0,}}}'

        # Variable
        matches = re.findall(regexVar, command)
        for match in matches:
            matchStr = match[2:-2]
            # varName = matchStr[matchStr.find(':') + 1:].strip()
            varName = matchStr.strip()
            command = command.replace(match, self.data.getVar(varName))
            self.log.log('> Injecting Variable \'' + varName + '\': ' + command)
            self.log.log('')

        # URL
        matches = re.findall(regexUrl, command)
        for match in matches:
            matchStr = match[2:-2]
            urlName = matchStr[matchStr.find(':') + 1:].strip()
            command = command.replace(match, self.data.getUrl(urlName))
            self.log.log('> Injecting URL \'' + urlName + '\': ' + command)
            self.log.log('')

        # Request
        matches = re.findall(regexRequest, command)
        for match in matches:
            matchStr = match[2:-2]
            reqName = matchStr[matchStr.find(':') + 1:].strip()
            # print(reqName)
            # print(json.dumps(self.data.dataStore))
            command = command.replace(match, self.data.getData(reqName))
            self.log.log('> Injecting Request \'' + reqName + '\': ' + command)
            self.log.log('')
        return command

    # Begins executing the script from top to bottom & handles nested scripts
    def handle(self, filepath):
        from command import Command
        if (self.debug):
            self.log.log('>>> Opening Script: ' + filepath[0])
            self.log.log('')
            self.log.log('')
        commandHandler = None
        with open(filepath[0]) as fileObj:
            i = 0
            self.request = {}
            self.request['text'] = {}
            for line in fileObj:
                if (line.strip() == ''):
                    continue
                line = ''.join(line.splitlines())

                self.log.log('> Now processing: ' + line)
                self.log.log('')

                self.evaluateScriptax(line)
                self.log.log('')

                # line = self.inject(line)

                # if(not self.detectScriptax(line, lineNumber = i)):
                #  commandHandler = Command(self.header, line, self.config, debug = self.debug, sensitive = self.sensitive)

                #  if(commandHandler.getRequest().getResponseBody().strip() != ''):
                # if(isinstance(commandHandler.getRequest(), Script)):
                # if(commandHandler.getRequest().data.name == ""):
                #  fileName = os.path.basename(line[6:].strip())
                #  if("." in fileName):
                #    fileName = fileName.split('.')[0]
                #  commandHandler.getRequest().data.name = fileName
                # scriptName = commandHandler.getRequest().data.name
                # self.data.store(commandHandler.getRequest().data.getRequestData()) # Imports the request data from embedded scripts - this should eventually be removed in favor of exports
                # print('THE EXPORTS:')
                # print(commandHandler.getRequest().data.getExports())
                # print(self.data.dataStore)
                # self.data.importExports(commandHandler.getRequest().data.getExports(), prefix=scriptName)
                # else:
                #  self.data.store(commandHandler.getRequest().getResponseBody())
                #    self.data.store(commandHandler.getRequest().getResponseBody())
                #    self.request['text'][i] = {"command": line.strip(), "status": commandHandler.getRequest().getResponseStatusCode(),"body":json.loads(commandHandler.getRequest().getResponseBody())}
                #  else:
                #    self.request['text'][i] = {"command": line.strip(), "status": commandHandler.getRequest().getResponseStatusCode(),"body":""}

                i += 1
            self.request['text'] = json.dumps(self.request['text'])
            self.request['status_code'] = 200

    def evaluateScriptax(self, line, injectorsOnly=False):
        line = ''.join(line.splitlines())
        while (True):
            result = self.scriptax(line.strip(), injectorsOnly=injectorsOnly)
            if (result is None):
                return line
            else:
                line = result

    def scriptax(self, line, injectorsOnly=False):

        if (line == ""):
            return None

        # INJECTORS

        # Variable
        matches = re.findall(self.regexVar, line)
        for match in matches:
            return self.stVariable(line, match)

        # URL
        matches = re.findall(self.regexUrl, line)
        for match in matches:
            return self.stUrl(line, match)

        # Request
        matches = re.findall(self.regexRequest, line)
        for match in matches:
            return self.stRequest(line, match)

        # Execute Commandtax
        matches = re.findall(self.regexExecuteCommandtax, line)
        for match in matches:
            return self.stExecuteCommandtax(line, match)

        # CONSUMERS

        if (not injectorsOnly):

            # Set Variable
            matches = re.findall(self.regexSetVar, line)
            for match in matches:
                return self.stSetVariable(line, match)

                # Set URL
            matches = re.findall(self.regexSetUrl, line)
            for match in matches:
                return self.stSetUrl(line, match)

                # Export Existing Variable
            matches = re.findall(self.regexExportVar, line)
            for match in matches:
                return self.stExportVariable(line, match)

                # Export New Variable
                # Needs more work, maybe a method for determining this
            matches = re.findall(self.regexExportNewVar, line)
            for match in matches:
                return self.stExportVariable(line, match)

                # Import
            matches = re.findall(self.regexImport, line)
            for match in matches:
                return self.stImport(line, match)

            # Export
            matches = re.findall(self.regexExportRequest, line)
            for match in matches:
                return self.stExportRequest(line, match)

                # Name
            matches = re.findall(self.regexName, line)
            for match in matches:
                return self.stName(line, match)

                # Commandtax
            matches = re.findall(self.regexCommandtax, line)
            for match in matches:
                return self.stCommandtax(line, match)

                # self.stExecuteCommandtax(line, line)

                # matches = re.findall(self.regexCommandtax, line)
                # for match in matches:
                #  return self.stExecuteCommandtax(line, match)

        return None

    # Consumer
    def stName(self, line, match):
        matchStr = (match[4:])
        name = matchStr.strip()
        name = self.evaluateScriptax(name)
        self.data.name = name
        if (self.debug):
            self.log.log('> Setting Script Name: ' + name)
            self.log.log('')
        return ""

    def stRequire(self, line, match):
        return ""

    # Consumer
    def stExecuteCommandtax(self, line, match):
        matchStr = match[2:-2]
        importCommand = self.evaluateScriptax(matchStr, injectorsOnly=True)
        response = self.executeCommand(importCommand, returnRequest=True)
        return response.getResponseBody()

    # Consumer
    def stCommandtax(self, line, match):
        matchStr = match[1:]
        importCommand = self.evaluateScriptax(matchStr, injectorsOnly=True)
        self.executeCommand(importCommand)
        return ""

    # Consumer
    def stSetVariable(self, line, match):
        matchStr = match[3:]
        var = matchStr.split('=')
        vName = self.evaluateScriptax(var[0].strip())
        vVal = self.evaluateScriptax(var[1].strip())
        self.data.storeVar(vName, vVal)
        if (self.debug):
            self.log.log('> Assigning Variable: ' + vName + ' = ' + vVal)
            self.log.log('')
        return ""

    # Consumer
    def stSetUrl(self, line, match):
        matchStr = match[2:-2]
        matchStr = matchStr[matchStr.find(':') + 1:].strip()
        matchStr = self.evaluateScriptax(matchStr)
        urlData = json.loads(matchStr)
        uName = urlData['name'].strip()
        uVal = urlData['url'].strip()
        self.data.storeUrl(uName, uVal)
        if (self.debug):
            self.log.log('> Assigning URL: ' + uName + ' = ' + uVal)
            self.log.log('')
        return ""

        # Consumer

    def stImport(self, line, match):
        matchStr = (match[6:])
        importCommand = matchStr.strip()
        importCommand = self.evaluateScriptax(importCommand)

        request = self.executeCommand(importCommand, returnRequest=True)

        self.importCommandRequest(importCommand, request)

        if (self.debug):
            self.log.log('> Importing: ' + importCommand)
            self.log.log('')

        return ""

    # Consumer
    def stExportVariable(self, line, match):
        matchStr = (match[6:])
        varName = matchStr.strip()
        self.data.exportVar(varName)
        if (self.debug):
            self.log.log('> Exporting Variable: ' + varName + ' = ' + self.data.getVar(varName))
            self.log.log('')
        return ""

    # Consumer
    def stExportRequest(self, line, match):
        matchStr = (match[6:])
        importCommand = matchStr.strip()
        importCommand = self.evaluateScriptax(importCommand)

        request = self.executeCommand(importCommand, returnRequest=True)

        scriptName = self.importCommandRequest(importCommand, request, export=True)

        # self.data.exportRequest(scriptName)

        if (self.debug):
            self.log.log('> Exporting Request: ' + importCommand)
            self.log.log('')

        return ""

    # Injecter
    def stRequest(self, line, match):
        matchStr = match[2:-2]
        reqName = matchStr[matchStr.find(':') + 1:].strip()
        reqName = self.evaluateScriptax(reqName)
        # print(reqName)
        replacer = self.data.getRequest(reqName)
        try:
            line = line.replace(match, replacer)
        except:
            self.log.log('')
            self.log.error('Injection Failure. Cannot access request for data: ' + json.dumps(replacer))
            self.log.log('')
            return ""
        self.log.log('> Injecting Request \'' + reqName + '\': ' + line)
        self.log.log('')
        return line

    # Injecter
    def stUrl(self, line, match):
        matchStr = match[2:-2]
        urlName = matchStr[matchStr.find(':') + 1:].strip()
        urlName = self.evaluateScriptax(urlname)
        replacer = self.data.getUrl(urlName)
        try:
            line = line.replace(match, replacer)
        except:
            self.log.log('')
            self.log.error('Injection Failure. Cannot access url for data: ' + json.dumps(replacer))
            self.log.log('')
            return ""
        self.log.log('> Injecting URL \'' + urlName + '\': ' + line)
        self.log.log('')
        return line

    # Injecter
    def stVariable(self, line, match):
        matchStr = match[2:-2]
        # varName = matchStr[matchStr.find(':') + 1:].strip()
        varName = matchStr.strip()
        varName = self.evaluateScriptax(varName)
        replacer = self.data.getVar(varName)
        try:
            line = line.replace(match, replacer)
        except:
            self.log.log('')
            self.log.error('Injection Failure. Cannot access variable for data: ' + json.dumps(replacer))
            self.log.log('')
            return ""
        self.log.log('> Injecting Variable \'' + varName + '\': ' + line)
        self.log.log('')
        return line

    def importCommandRequest(self, command, request, export=False):

        scriptName = ""

        if (request.getResponseBody().strip() != ''):

            if (isinstance(request, Script)):

                if (request.data.name == ""):
                    # Gets the name of the script file when a name scriptax is not specified
                    fileName = os.path.basename(command[6:].strip())
                    if ("." in fileName):
                        fileName = fileName.split('.')[0]
                    request.data.name = fileName

                scriptName = request.data.name

                # self.data.store(commandHandler.getRequest().data.getRequestData(), prefix=scriptName)
                self.data.importScriptsExports(request.data, prefix=scriptName, export=export)
                # print(json.dumps(self.data.dataStore))
            else:
                self.data.storeRequest(request.getResponseBody(), export=export)
                # self.data.store(commandHandler.getRequest().getResponseBody())
            # self.request['text'][lineNumber] = {"command": importCommand.strip(), "status": commandHandler.getRequest().getResponseStatusCode(),"body":json.loads(commandHandler.getRequest().getResponseBody())}
        else:
            pass
            # self.request['text'][lineNumber] = {"command": importCommand.strip(), "status": commandHandler.getRequest().getResponseStatusCode(),"body":""}

        return scriptName

    def executeCommand(self, importCommand, returnRequest=False):
        from command import Command

        commandHandler = Command(self.header, importCommand, self.config, debug=self.debug, sensitive=self.sensitive)

        if (returnRequest):
            return commandHandler.getRequest()
            # return json.dumps(dict({"command": importCommand.strip(), "status": commandHandler.getRequest().getResponseStatusCode(),"body":json.loads(commandHandler.getRequest().getResponseBody())}))

    # return json.dumps({"command": importCommand.strip(), "status": commandHandler.getRequest().getResponseStatusCode(),"body":json.loads(commandHandler.getRequest().getResponseBody())})



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
  name = 'apitax',
  packages = find_packages(), # this must be the same as the name above
  version = '2.0.1',
  description = 'Apitax combines the power of Scriptax and COmmandtax into a quick and easy to use Python package to facillitate powerful Restful API Request Scripting',
  long_description=long_description,
  long_description_content_type="text/markdown",
  author = 'Shawn Clake',
  author_email = 'shawn.clake@gmail.com',
  url = 'https://github.com/ShawnClake/Apitax', # use the URL to the github repo
  #download_url = '', # I'll explain this in a second
  keywords = ['restful', 'api', 'commandtax', 'scriptax'], # arbitrary keywords
  classifiers = (
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
  ),
  install_requires = [
    'click',
    'bottle',
    'requests',
    'antlr4-python3-runtime',
  ],
)


# Generated from D:/Programming/Projects/NoPatience/API-Middleware/grammar/src\test1.g4 by ANTLR 4.7
from antlr4 import *

if __name__ is not None and "." in __name__:
    from .test1Parser import test1Parser
else:
    from test1Parser import test1Parser


# This class defines a complete listener for a parse tree produced by test1Parser.
class test1Listener(ParseTreeListener):

    # Enter a parse tree produced by test1Parser#operation.
    def enterOperation(self, ctx: test1Parser.OperationContext):
        pass

    # Exit a parse tree produced by test1Parser#operation.
    def exitOperation(self, ctx: test1Parser.OperationContext):
        pass



# System Imports
import json
import re

# Application import
from apitax.logs.Log import Log


# Container for all the data returned by a sequence of script requests
class ScriptData:
    def __init__(self):
        self.dataStore = dict(
            {"urls": {}, "vars": {}, "requests": {}, "exports": {"urls": {}, "vars": {}, "requests": {}}}
        )
        self.name = ""
        self.index = 1
        # self.log = Log('logs/log.log')

    # Used for current script interactions
    def storeUrl(self, name, url):
        key = "urls." + name
        self.storeDotNotation(url, key)
        if (self.isExistDotNotation("exports." + key)):
            self.exportUrl(name)

    def getUrl(self, name):
        key = "urls." + name
        return self.getDotNotation(key)

    # Used for current script interactions
    def storeVar(self, name, var):
        key = "vars." + name
        self.storeDotNotation(var, key)
        if (self.isExistDotNotation("exports." + key)):
            self.exportVar(name)

    def getVar(self, name):
        key = "vars." + name
        return self.getDotNotation(key)

    # Used for current script interactions
    def storeRequest(self, data, export=False):
        key = "requests." + str(self.index)
        self.storeDotNotation(data, key)
        if (export):
            self.exportRequest(str(self.index))
        self.index += 1
        # if(isinstance(data, dict)):
        #  self.dataStore[str(self.index)] = data
        # else:
        #  self.dataStore[str(self.index)] = json.loads(data)
        # if(prefix == ""):
        #  prefix = self.name
        # if(prefix != ""):
        #  prefix += '.'
        # if(isinstance(data, dict)):
        # self.dataStore[prefix+str(self.index)] = data
        #  self.dataStore[len(self.dataStore)] = data
        # else:
        # self.dataStore[prefix+str(self.index)] = json.loads(data)
        #  self.dataStore[len(self.dataStore)] = json.loads(data)

        # self.log.log(self.dataStore)

    def getRequest(self, index):
        key = "requests." + str(index)
        return self.getDotNotation(key)

    def getRequestData(self):
        newData = self.dataStore.copy()
        # newData.pop("0")
        return newData

    def importVar(self, data, dottedKey):
        key = "vars." + dottedKey
        self.storeDotNotation(data, key)

    def importRequest(self, data, dottedKey):
        key = "requests." + dottedKey
        self.storeDotNotation(data, key)

    def importUrl(self, data, dottedKey):
        key = "urls." + dottedKey
        self.storeDotNotation(data, key)

    def exportVar(self, dottedVarName):
        key = "exports.vars." + dottedVarName
        self.storeDotNotation(self.getVar(dottedVarName), key)
        # self.dataStore["exports"]["vars"][varName] = self.getVar(varName)

    def getVarExports(self):
        return self.getDotNotation("exports.vars")

    def exportRequest(self, dottedRequestName):
        key = "exports.requests." + dottedRequestName
        self.storeDotNotation(self.getRequest(dottedRequestName), key)

    def getRequestExports(self):
        return self.getDotNotation("exports.requests")

    def exportUrl(self, dottedUrlName):
        key = "exports.urls." + dottedUrlName
        self.storeDotNotation(self.getVar(dottedUrlName), key)

    def getUrlExports(self):
        return self.getDotNotation("exports.urls")

    def getExports(self):
        return self.getDotNotation("exports")

    def storeDotNotation(self, data, key):
        components = key.split('.')

        finalIndex = components.pop()
        navigation = self.dataStore

        for component in components:
            if (component not in navigation):
                if (isinstance(navigation, list)):
                    navigation[int(component)] = {}
                else:
                    navigation[component] = {}
            if (isinstance(navigation, list)):
                navigation = navigation[int(component)]
            else:
                navigation = navigation[str(component)]
            # navigation = navigation[component]

        if (finalIndex not in navigation):
            navigation[finalIndex] = {}

        if (self.isJson(data)):
            data = json.loads(data)

        navigation[finalIndex] = data

    def getDotNotation(self, key):
        components = key.split('.')
        navigation = self.dataStore
        for component in components:
            # if(isinstance(navigation, list) and int(component) not in navigation):
            #  navigation[int(component)] = {}

            if(navigation is None):
                raise ValueError('Variable does not exist | Variable access is undefined')

            if (component not in navigation and not isinstance(navigation, list)):
                pass
                # navigation[component] = {}
                # if(isinstance(navigation, list)):
                #  navigation[int(component)] = {}
                # else:
                #  navigation[component] = {}
            if (isinstance(navigation, list)):
                navigation = navigation[int(component)]
            else:
                navigation = navigation[str(component)]
            # navigation = navigation[component]
        return navigation

    def isExistDotNotation(self, key):
        components = key.split('.')
        navigation = self.dataStore
        for component in components:
            if (component not in navigation and not isinstance(navigation, list)):
                return False
            if (isinstance(navigation, list)):
                navigation = navigation[int(component)]
            else:
                navigation = navigation[str(component)]
            # navigation = navigation[component]
        return True

    def importScriptsExports(self, scriptData, prefix="", export=False):
        if (prefix == ""):
            prefix = scriptData.name
        if (prefix != ""):
            prefix += '.'

        # print(scriptData.dataStore)

        for varName, varVal in scriptData.getVarExports().items():
            self.importVar(varVal, prefix + varName)
            if (export):
                self.exportVar(prefix + varName)

        for urlName, urlVal in scriptData.getUrlExports().items():
            self.importUrl(urlVal, prefix + urlName)
            if (export):
                self.exportUrl(prefix + urlName)

        for requestIndex, request in scriptData.getRequestExports().items():
            self.importRequest(request, prefix + requestIndex)
            if (export):
                self.exportRequest(prefix + requestIndex)

    # DotString is the form   7.someIndex.someOther.andTheLast
    # def getData(self, dotString):
    #  components = dotString.split('.')
    #  response = self.getResponse(components.pop(0))
    #  for identifier in components:
    #    if(isinstance(response, list)):
    #      response = response[int(identifier)]
    #    else:
    #      response = response[str(identifier)]
    #  #self.log.log(response)
    #  return response

    def isJson(self, x):
        try:
            json.loads(x)
            return True
        except:
            return False

    def isJsonable(self, x):
        try:
            json.dumps(x)
            return True
        except:
            return False



# Generated from D:/Programming/Projects/Apitax/grammar/src\Ah2.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .Ah2Parser import Ah2Parser
else:
    from Ah2Parser import Ah2Parser

# This class defines a complete generic visitor for a parse tree produced by Ah2Parser.

class Ah2Visitor(ParseTreeVisitor):

    # Visit a parse tree produced by Ah2Parser#prog.
    def visitProg(self, ctx:Ah2Parser.ProgContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#statements.
    def visitStatements(self, ctx:Ah2Parser.StatementsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#statement.
    def visitStatement(self, ctx:Ah2Parser.StatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#expr.
    def visitExpr(self, ctx:Ah2Parser.ExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#set_var.
    def visitSet_var(self, ctx:Ah2Parser.Set_varContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#scoping.
    def visitScoping(self, ctx:Ah2Parser.ScopingContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#name.
    def visitName(self, ctx:Ah2Parser.NameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#exports.
    def visitExports(self, ctx:Ah2Parser.ExportsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#imports.
    def visitImports(self, ctx:Ah2Parser.ImportsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#execute.
    def visitExecute(self, ctx:Ah2Parser.ExecuteContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#inject.
    def visitInject(self, ctx:Ah2Parser.InjectContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Ah2Parser#variable_types.
    def visitVariable_types(self, ctx:Ah2Parser.Variable_typesContext):
        return self.visitChildren(ctx)



del Ah2Parser